/***********************************************************
        Copyright 1991,1994 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/


#ifndef SABER
#ifndef LINT
static char rcs_id[]="$Id: Hint_CollectAllHints.c,v 1.4 1994/08/12 18:02:52 ww0r Exp $";
#endif /* LINT */
#endif /* SABER */

/*
 * Author: Sohan C. Ramakrishna Pillai
 */

#include "depotlib.h"

#include "util.h"
#include "DepotErrorCodes.h"
#include "File.h"
#include "FileSystemImage.h"
#include "Hint.h"

static HINTDB Hint_Tree_CollectAllHints();

HINTDB Hint_CollectAllHints(fsimage)
     FILESYSTEMIMAGE *fsimage;
{
  HINTDB hintinfo;

  hintinfo = NULL;

  /* recurse to collect hints */
  hintinfo = Hint_Tree_CollectAllHints(hintinfo, fsimage, ""/*pathtonode*/);

  return (PROGRAM_ErrorNo == E_NULL) ? hintinfo : NULL;
}


static HINTDB Hint_Tree_CollectAllHints(hintinfo, fsimage, pathtonode)
     HINTDB hintinfo;
     FILESYSTEMIMAGE *fsimage;
     char *pathtonode;
{
  register FILESYSTEMIMAGE *fsp;
  HINTDB hp;
  HINT hint;
  char hintpath[MAXPATHLEN], id[MAXPATHLEN];

  hp = hintinfo;
  HINT_PathName(&hint) = hintpath;
  HINT_UniqueID(&hint) = id;

  for (fsp = fsimage; fsp != NULL; fsp = FILESYSTEMIMAGE_Next(fsp))
    {
      if (FILESYSTEMIMAGE_Type(fsp) & FS_DIR)
	{
	  if ((pathtonode[0] == '\0')
	      || (strcmp(pathtonode, "/") == 0)
	      || (strcmp(pathtonode, ".") == 0))
	    (void)strcpy(HINT_PathName(&hint), FILESYSTEMIMAGE_Name(fsp));
	  else if ((strcmp(FILESYSTEMIMAGE_Name(fsp), "/") == 0)
		   || (strcmp(FILESYSTEMIMAGE_Name(fsp), ".") == 0))
	    (void)strcpy(HINT_PathName(&hint), pathtonode);
	  else
	    (void)sprintf(HINT_PathName(&hint), "%s/%s",
			  pathtonode, FILESYSTEMIMAGE_Name(fsp));
	  if ((FILESYSTEMIMAGE_Type(fsp) & FS_FSMOUNT)
	      && (FILESYSTEMIMAGE_FSMountID(fsp) != NULL))
	    {
	      (void)sprintf(HINT_UniqueID(&hint),
			    FILESYSTEMIMAGE_FSMountID(fsp));
	      HINT_ModTime(&hint) = FILESYSTEMIMAGE_DTime(fsp);
	      hp = HintDB_InsertHint(hp, &hint, FALSE/*!OverWrite*/);
	    }
	  hp = Hint_Tree_CollectAllHints(hp,
					 FILESYSTEMIMAGE_Child(fsp),
					 HINT_PathName(&hint));
	}
    }

  return (PROGRAM_ErrorNo == E_NULL) ? hp : NULL;
}

/* $Source: /afs/andrew.cmu.edu/system/src/local/depot2/011/src/lib/Hint/RCS/Hint_CollectAllHints.c,v $ */
